package io.chaoge.shareview;


import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.os.Environment;
import android.provider.MediaStore;
import android.text.TextUtils;
import android.widget.Toast;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.ReadableArray;
import com.facebook.react.common.MapBuilder;
import com.facebook.react.uimanager.ThemedReactContext;
import com.facebook.react.uimanager.UIManagerModule;
import com.facebook.react.uimanager.events.EventDispatcher;
import com.facebook.react.views.view.ReactViewGroup;
import com.facebook.react.views.view.ReactViewManager;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

import javax.annotation.Nullable;

public class ShareViewBridgeManager extends ReactViewManager {
  private static final String REACT_CLASS = "RCTShareView";

  public static final int SHARE_PIC = 103;
  public static final int SAVE_PIC = 104;

  private static final String SAVE_PIC_PATH= Environment.getExternalStorageState().equalsIgnoreCase(Environment.MEDIA_MOUNTED) ? Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DCIM).getAbsolutePath()+"/Camera" : "";
  private EventDispatcher mEventDispatcher;
  private ReactContext context;

  public ShareViewBridgeManager(ReactApplicationContext reactApplicationContext) {
    super();
  }
  @Override
  public ReactViewGroup createViewInstance(ThemedReactContext reactContext) {
    context=reactContext;
    mEventDispatcher=reactContext.getNativeModule(UIManagerModule.class).getEventDispatcher();
    return super.createViewInstance(reactContext);
  }

  @Override
  public String getName() {
    return REACT_CLASS;
  }

  @Nullable
  @Override
  public Map<String, Object> getExportedCustomBubblingEventTypeConstants() {
    return MapBuilder.<String, Object>builder()
            .put("topSharePrepared",MapBuilder.of(
                    "phasedRegistrationNames",
                    MapBuilder.of("bubbled", "onSharePrepared", "captured", "onSharePreparedCapture")))

            .build();
  }
  @Override
  public @Nullable Map<String, Integer> getCommandsMap() {
    Map<String, Integer> commandsMap = super.getCommandsMap();
    commandsMap.put("sharePic",SHARE_PIC);
    commandsMap.put("savePic",SAVE_PIC);
    return  commandsMap;
  }

    @Override
    public void receiveCommand(ReactViewGroup root, int commandId, @Nullable ReadableArray args) {
        super.receiveCommand(root, commandId, args);

    switch (commandId) {
      case SHARE_PIC:
        Share(root);
        break;
      case SAVE_PIC:
        Save(root);
        break;
      default:
        //do nothing!!!!
    }
  }

  /**
   * 分享
   */
  public void Share(ReactViewGroup view){
    if (TextUtils.isEmpty(SAVE_PIC_PATH))
      return;
        try {
          final File realFile = saveBitmap("share.jpg",view);
          if (realFile == null){
            if(mEventDispatcher!=null){
              mEventDispatcher.dispatchEvent(new SharePreparedEvent(view.getId(),realFile.getAbsolutePath(),"分享失败;文件过大"));
            }
          }else{
            if(mEventDispatcher!=null){
              mEventDispatcher.dispatchEvent(new SharePreparedEvent(view.getId(),realFile.getAbsolutePath(),""));
            }
          }
        }catch (Exception e){
          e.printStackTrace();
          if(mEventDispatcher!=null){
            mEventDispatcher.dispatchEvent(new SharePreparedEvent(view.getId(),"","分享失败"));
          }
        }finally {
        }
  }

  /**
   * 保存
   */
  public void Save(ReactViewGroup view){
    if (TextUtils.isEmpty(SAVE_PIC_PATH))
      return;
        try {
          SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd_hhmmss");
          final File realFile = saveBitmap(sdf.format(new Date())+".jpg",view);
          if (realFile == null){
            Toast.makeText(context,"保存失败,文件过大!",Toast.LENGTH_SHORT).show();
          }else{
            // 其次把文件插入到系统图库
            try {
              MediaStore.Images.Media.insertImage(context.getContentResolver(),
                      realFile.getAbsolutePath(), realFile.getName(), null);
            } catch (FileNotFoundException e) {
              e.printStackTrace();
            }
            realFile.delete();
            //data.putString("path",realFile.getAbsolutePath());
            Toast.makeText(context,"图片已保存在相册!",Toast.LENGTH_LONG).show();
          }
        } catch (final Exception e) {
          e.printStackTrace();
          Toast.makeText(context,"保存失败!",Toast.LENGTH_SHORT).show();
        }finally {
        }
  }
  /**
   * 截屏
   *
   * @return
   */
  public Bitmap getScreen(ReactViewGroup view) {
    Bitmap bmp = Bitmap.createBitmap(view.getWidth(), 1, Bitmap.Config.ARGB_8888);
    int rowBytes = bmp.getRowBytes();
    bmp = null;

    if (rowBytes*view.getHeight()>=getAvailMemory()){
      return null;
    }
    bmp = Bitmap.createBitmap(view.getWidth(), view.getHeight(), Bitmap.Config.ARGB_8888);
    Canvas canvas = new Canvas(bmp);
    view.draw(canvas);
    return bmp;
  }
  private long getAvailMemory() {// 获取android当前可用内存大小
    return Runtime.getRuntime().maxMemory();
  }
  /**
   * 保存图片到文件
   * @param fileName  文件名称
   * @return
   * @throws Exception
   */
  private File saveBitmap(String fileName,ReactViewGroup view) throws Exception{
      Bitmap bitmap = getScreen(view);
      if (bitmap == null)
        return null;
      File file = new File(SAVE_PIC_PATH);
      if (!file.exists()) {
        file.mkdirs();
      }
      final File realFile = new File(file, fileName);
      if (!realFile.exists()){
        realFile.createNewFile();
      }
      FileOutputStream fos = new FileOutputStream(realFile);
      bitmap.compress(Bitmap.CompressFormat.JPEG, 100, fos);
      fos.flush();
      fos.close();
      if (!bitmap.isRecycled()) {
        bitmap.recycle();
        System.gc(); // 通知系统回收
      }
      return realFile;
    }


}
