/**
 * Copyright (c) 2015-present, Facebook, Inc.
 * All rights reserved.
 *
 * This source code is licensed under the BSD-style license found in the
 * LICENSE file in the root directory of this source tree. An additional grant
 * of patent rights can be found in the PATENTS file in the same directory.
 *
 * @providesModule View
 * @flow
 */
'use strict';

// const Platform = require('Platform');
// const EdgeInsetsPropType = require('EdgeInsetsPropType');
// const NativeMethodsMixin = require('NativeMethodsMixin');
// const NativeModules = require('NativeModules');

import PropTypes from 'prop-types'

const React = require('react');
const ReactNative=require('react-native')

import {UIManager,Platform,EdgeInsetsPropType,NativeMethodsMixin,
    NativeModules,
    ViewStylePropTypes,
    ViewAccessibility,
    View,
    requireNativeComponent,
} from 'react-native'
const invariant = require('invariant');
const {
    AccessibilityComponentTypes,
    AccessibilityTraits,
}=require('../react-native/Libraries/Components/View/ViewAccessibility')
const ReactNativeViewAttributes = require('../react-native/Libraries/Components/View/ReactNativeViewAttributes');
const ReactNativeStyleAttributes = require('../react-native/Libraries/Components/View/ReactNativeStyleAttributes');

const StyleSheetPropType = require('../react-native/Libraries/StyleSheet/StyleSheetPropType');

var TVViewPropTypes = {};
if (Platform.isTVOS) {
    //TVViewPropTypes = require('TVViewPropTypes');
    TVViewPropTypes=ReactNative.TVViewPropTypes
}

//const requireNativeComponent = require('requireNativeComponent');

// const PropTypes = React.PropTypes;

const stylePropType = StyleSheetPropType(ViewStylePropTypes);

const forceTouchAvailable = (NativeModules.PlatformConstants &&
    NativeModules.PlatformConstants.forceTouchAvailable) || false;

const statics = {
    AccessibilityTraits,
    AccessibilityComponentType: AccessibilityComponentTypes,

    forceTouchAvailable,
};

var createReactClass = require('create-react-class');

const ShareView = createReactClass({
    mixins: [NativeMethodsMixin],
    viewConfig: {
        uiViewClassName: 'RCTShareView',
        validAttributes: ReactNativeViewAttributes.RCTView
    },

    statics: {
        ...statics,
    },

    propTypes: {
        ...TVViewPropTypes,
        accessible: PropTypes.bool,
        accessibilityLabel: PropTypes.node,
        accessibilityComponentType: PropTypes.oneOf(AccessibilityComponentTypes),
        accessibilityLiveRegion: PropTypes.oneOf([
            'none',
            'polite',
            'assertive',
        ]),

        importantForAccessibility: PropTypes.oneOf([
            'auto',
            'yes',
            'no',
            'no-hide-descendants',
        ]),


        accessibilityTraits: PropTypes.oneOfType([
            PropTypes.oneOf(AccessibilityTraits),
            PropTypes.arrayOf(PropTypes.oneOf(AccessibilityTraits)),
        ]),

        accessibilityViewIsModal: PropTypes.bool,
        onAccessibilityTap: PropTypes.func,


        onMagicTap: PropTypes.func,

        testID: PropTypes.string,

        onResponderGrant: PropTypes.func,

        onResponderMove: PropTypes.func,


        onResponderReject: PropTypes.func,

        onResponderRelease: PropTypes.func,

        onResponderTerminate: PropTypes.func,

        onResponderTerminationRequest: PropTypes.func,

        onStartShouldSetResponder: PropTypes.func,


        onStartShouldSetResponderCapture: PropTypes.func,


        onMoveShouldSetResponder: PropTypes.func,

        onMoveShouldSetResponderCapture: PropTypes.func,


        hitSlop: EdgeInsetsPropType,

        onLayout: PropTypes.func,


        pointerEvents: PropTypes.oneOf([
            'box-none',
            'none',
            'box-only',
            'auto',
        ]),
        style: stylePropType,


        removeClippedSubviews: PropTypes.bool,


        renderToHardwareTextureAndroid: PropTypes.bool,


        shouldRasterizeIOS: PropTypes.bool,

        collapsable: PropTypes.bool,


        needsOffscreenAlphaCompositing: PropTypes.bool,
    },

    contextTypes: {
        isInAParentText: PropTypes.bool,
    },

    render: function() {
        invariant(
            !(this.context.isInAParentText && Platform.OS === 'android'),
            'Nesting of <View> within <Text> is not supported on Android.');
        return <RCTShareView ref={"_inputRef"} {...this.props} onSharePrepared={this.onSharePrepared}/>;
    },
    getViewHandle: function() {
        return ReactNative.findNodeHandle(this.refs._inputRef);
    },
    //分享图片生成成功后的回调
    onSharePrepared:function(event){
        if (this.props.onSharePrepared != null && event.nativeEvent != null) {
            this.props.onSharePrepared(event.nativeEvent.path,event.nativeEvent.errmsg)
        }
    },
    //生成分享图片
    sharePic: function() {
        if(Platform.OS=="ios"){
            ReactNative.takeSnapshot(this.getViewHandle(), {format: 'jpeg', quality: 1.0}).then(
                (uri) => {
                    this.props.onSharePrepared(uri,"")
                }
            ).catch(
                (error) => {
                    this.props.onSharePrepared(null,error)
                }
            );
        }else{
            UIManager.dispatchViewManagerCommand(
                this.getViewHandle(),
                UIManager.RCTShareView.Commands.sharePic,
                null
            );
        }
    },
    //保存图片
    savePic: function() {
        UIManager.dispatchViewManagerCommand(
            this.getWebViewBridgeHandle(),
            UIManager.RCTShareView.Commands.savePic,
            null
        );
    },

});

const RCTShareView =null;
if(Platform.OS == "ios"){
    RCTShareView = View;
}else{
    RCTShareView = requireNativeComponent('RCTShareView', ShareView, {
        nativeOnly: {
            nativeBackgroundAndroid: true,
            nativeForegroundAndroid: true,
        }
    });
}


if (__DEV__) {
    //const UIManager = require('UIManager');
    const viewConfig = UIManager.viewConfigs && UIManager.viewConfigs.RCTShareView || {};
    for (const prop in viewConfig.nativeProps) {
        const viewAny: any = ShareView; // Appease flow
        if (!viewAny.propTypes[prop] && !ReactNativeStyleAttributes[prop]) {
            throw new Error(
                'View is missing propType for native prop `' + prop + '`'
            );
        }
    }
}

let ViewToExport = RCTShareView;
if (__DEV__) {
    ViewToExport = ShareView;
} else {
    ViewToExport = ShareView;
}

module.exports = ViewToExport;
